# tcp_cmd

tcp_cmd is the heart of the operation. It is in charge of handling the
connection state for all connections, as well as reacting to events
from the user as well as incoming packets. It is defined by the TCP
spec RFC 793.

![tcp_cmd_drawio.png](tcp_cmd.drawio.png)

## SW Interface

The software interface is based on array of register files, where
commands/data is transferred by reading or writing to a register.
See the register docs for more information.

## HW Interfaces

tcp_cmd communicates with both tcp_tx and tcp_rx with a uni-directional
axi stream bus.

tcp_rx -> tcp_cmd -> tcp_tx

Signals that need to go from tcp_cmd to tcp_rx or from tcp_tx to tcp_cmd
must be separate signals. The number of these signals should be minimized.

## TCB
The TCP spec specifices a "TCB", or Transmission Control Block. The TCB
stores information about connections and is how we will support multiple
connections with the same hardware.

The TCB stores the following
- Local Socket Number           16 bits
- Remote Socket Number          16 bits
- Send pointer                  10 bits
- Recieve Pointer               10 bits
- Retransmit Queue Pointer      8 bits
- User Timer                    8 bits
- Retransmit Timer              8 bits
- Time-Wait Timer               8 bits
- Current state                 8 bits

the pointers are all 2kb addresses, and the data must be 2kb aligned. This is
to save bits in the memory. 72 bits total

Whenever an event occurs, 1 TCB is chosen as the active TCB. Only 1 connection
can be processed at a time, so this processing time must be less than the time
it takes to transmit or receive a packet. tx and rx are pipelined, so they can
receieve or transmit interleaved packets whithout caring about the active TCB.

The TCBs are stored as registers in the design. They could be stored in memories
to increase the number of active connections, but we must iterate through the TCB
whenever we recieve a packet. If we were to store the TCBs in memory, we would
need to load them one after another comparing the dest port until we find one
that matches. This would add a lot of latency and slow down how quickly we can
process incoming packets, and it is crucial that we do not drop incoming packets.

## Events

The TCP connection is defined by events. According to the spec, there are 10
events:

- OPEN
- SEND
- RECEIVE
- CLOSE
- ABORT
- STATUS
- SEGMENT ARRIVES
- USER TIMEOUT
- RETRANSMISSION TIMEOUT
- TIME-WAIT TIMEOUT

What happens during each event depends on which state we are in. I won't list
this here as it is in the spec (See page 52).

## Actions

- Create a TCB
- Send a SYN packet
- Change TCP State
- Send Data
- Send ACK
- Queue received data
- Delete a TCB
- Send a RST packet
- Return TCB data
- Drop incoming packet
- Retransmit packet
- Enable Timer

## TCB Memory

Each TCB is stored as one line in memory. The memory will be cycled continously
and each tcb sent to the event processor one after the other. The event handler
will determine if an event has happened for that connection. If so, it will
determine what action needs to be taken and send that action along with the
TCB to the action handler, and the event processor will read another TCB.

The action handler will use the TCB to get sequence numbers, pointers, etc. It
will also update the TCB and then write it back to memory. The TCB memory is
set up sort of like a FIFO where we will not read back a TCB twice in a row,
though that shouldn't really be possible anyway unless there is only 1 connection

## Event Processor

The event processor takes in a TCB as well as a bunch of signals from different
sources. These include the SW interface for OPEN, SEND, RECEIVE, CLOSE, ABORT,
and STATUS. The RX interface supplies SEGMENT ARRIVES, and the timers supply
the current timer value which is combined with the values in the TCB to generate
USER TIMEOUT, RETRANSMISSION TIMEOUT, and TIME-WAIT TIMEOUT.

Based on which event has occured and which state we are in, we choose an action
to take

## TCP RX Stream Interface

The TCP RX Stream Interface turns the axi stream interface between tcp_cmd and
tcp_rx into the event and flags needed for tcp_cmd

## TCP TX Stream Interface

The TCP TX Stream Interface turns the action output into the axi stream needed
for tcp_rx