# SW Interface

The SW interface exposes a register interface to the CPU and is how software
can create events, and actions that affect the sw interface will come here as
well.

It has an APB interface for the CPU.

![tcp_sw_interface.drawio.png](tcp_sw_interface.drawio.png)

## APB Interface

The APB interface looks like a series of registers. See the RDL for what this
looks like. The control/status registers work like normal registers, nothing
special here.

The stream registers are organized by event type. There are 6 events that the
CPU can trigger:
- OPEN
- SEND
- RECEIVE
- CLOSE
- ABORT
- STATUS

Due to the way it works, most of the registers will be write only or read only.

## Control Registers

The control registers are global, and they include things like enable/disable,
and the ability to change the number of TCP streams at runtime. This can reduce
latency if not all connections are being used.

## Stream Registers

### Open (Page 45)

"Format: OPEN (local port, foreign socket, active/passive
[, timeout] [, precedence] [, security/compartment] [, options])
-> local connection name"

The most important part is the local port and the foreign port.
The other fields are not as important.

Instead of returning a local connection name, this is already
specified since we wrote to a specific tcb index. This is used
instead of a specific name.

There is also a status register where you can read if the
even has been processed.

### Send (Page 46)

"Format: SEND (local connection name, buffer address, byte
count, PUSH flag, URGENT flag [,timeout])"

Once again in this case the local connection name is implied
by the address.

These are pretty obvious.

There is also a status register where you can read if the
even has been processed.

### RECEIVE (Page 48)

"Format: RECEIVE (local connection name, buffer address, byte
count) -> byte count, urgent flag, push flag"

In order to support higher throughput, The receive event will
be queued up even if there is no data ready to recieve. The
byte count and flags will be prepended to the data when it is
written.

The number of queued RECV calls is parameterizable.

The way receive works is that it will wait until there is enough
data in the buffer to fill up the request, unless the bytes are
associated with a PSH flag, in which case less than the remaining
bytes might be returned.

### CLOSE (Page 49)

"Format: CLOSE (local connection name)"

Simple close the connection.

### STATUS (Page 49)

"Format: STATUS (local connection name) -> status data"

The status is implemented by writing the status to a buffer
specified over DMA. This is to reduce address space usage.

### ABORT (Page 50)

"Format: ABORT (local connection name)"

Same as close but sends an abort instead of triggering the close sequence.


## Interface with Event Processor

All of the data needs to be passed to the event processor, but most of
it will just be forwarded to the action handler anyway. The widest the data
can be is during the open call, which takes 33 bits. We can make the bit
width 64 for expandability.