// Generated by PeakRDL-regblock - A free and open-source SystemVerilog generator
//  https://github.com/SystemRDL/PeakRDL-regblock

module tcp_cmd_regs (
        input wire clk,
        input wire rst,

        input wire s_apb_psel,
        input wire s_apb_penable,
        input wire s_apb_pwrite,
        input wire [11:0] s_apb_paddr,
        input wire [31:0] s_apb_pwdata,
        output logic s_apb_pready,
        output logic [31:0] s_apb_prdata,
        output logic s_apb_pslverr,

        input tcp_cmd_regs_pkg::tcp_cmd_regs__in_t hwif_in,
        output tcp_cmd_regs_pkg::tcp_cmd_regs__out_t hwif_out
    );

    //--------------------------------------------------------------------------
    // CPU Bus interface logic
    //--------------------------------------------------------------------------
    logic cpuif_req;
    logic cpuif_req_is_wr;
    logic [11:0] cpuif_addr;
    logic [31:0] cpuif_wr_data;
    logic [31:0] cpuif_wr_biten;
    logic cpuif_req_stall_wr;
    logic cpuif_req_stall_rd;

    logic cpuif_rd_ack;
    logic cpuif_rd_err;
    logic [31:0] cpuif_rd_data;

    logic cpuif_wr_ack;
    logic cpuif_wr_err;

    // Request
    logic is_active;
    always_ff @(posedge clk) begin
        if(rst) begin
            is_active <= '0;
            cpuif_req <= '0;
            cpuif_req_is_wr <= '0;
            cpuif_addr <= '0;
            cpuif_wr_data <= '0;
        end else begin
            if(~is_active) begin
                if(s_apb_psel) begin
                    is_active <= '1;
                    cpuif_req <= '1;
                    cpuif_req_is_wr <= s_apb_pwrite;
                    cpuif_addr <= {s_apb_paddr[11:2], 2'b0};
                    cpuif_wr_data <= s_apb_pwdata;
                end
            end else begin
                cpuif_req <= '0;
                if(cpuif_rd_ack || cpuif_wr_ack) begin
                    is_active <= '0;
                end
            end
        end
    end
    assign cpuif_wr_biten = '1;

    // Response
    assign s_apb_pready = cpuif_rd_ack | cpuif_wr_ack;
    assign s_apb_prdata = cpuif_rd_data;
    assign s_apb_pslverr = cpuif_rd_err | cpuif_wr_err;

    logic cpuif_req_masked;
    logic external_req;
    logic external_pending;
    logic external_wr_ack;
    logic external_rd_ack;
    always_ff @(posedge clk) begin
        if(rst) begin
            external_pending <= '0;
        end else begin
            if(external_req & ~external_wr_ack & ~external_rd_ack) external_pending <= '1;
            else if(external_wr_ack | external_rd_ack) external_pending <= '0;
            assert(!external_wr_ack || (external_pending | external_req))
                else $error("An external wr_ack strobe was asserted when no external request was active");
            assert(!external_rd_ack || (external_pending | external_req))
                else $error("An external rd_ack strobe was asserted when no external request was active");
        end
    end

    // Read & write latencies are balanced. Stalls not required
    // except if external
    assign cpuif_req_stall_rd = external_pending;
    assign cpuif_req_stall_wr = external_pending;
    assign cpuif_req_masked = cpuif_req
                            & !(!cpuif_req_is_wr & cpuif_req_stall_rd)
                            & !(cpuif_req_is_wr & cpuif_req_stall_wr);

    //--------------------------------------------------------------------------
    // Address Decode
    //--------------------------------------------------------------------------
    typedef struct {
        struct {
            logic active_streams;
            logic max_streams;
        } ctrl_regs;
        logic tcp_streams[16];
    } decoded_reg_strb_t;
    decoded_reg_strb_t decoded_reg_strb;
    logic decoded_strb_is_external;

    logic [11:0] decoded_addr;

    logic decoded_req;
    logic decoded_req_is_wr;
    logic [31:0] decoded_wr_data;
    logic [31:0] decoded_wr_biten;

    always_comb begin
        automatic logic is_external;
        is_external = '0;
        decoded_reg_strb.ctrl_regs.active_streams = cpuif_req_masked & (cpuif_addr == 12'h0);
        decoded_reg_strb.ctrl_regs.max_streams = cpuif_req_masked & (cpuif_addr == 12'h4);
        for(int i0=0; i0<16; i0++) begin
            decoded_reg_strb.tcp_streams[i0] = cpuif_req_masked & (cpuif_addr >= 12'h80 + i0*12'h80) & (cpuif_addr <= 12'h80 + i0*12'h80 + 12'h53);
            is_external |= cpuif_req_masked & (cpuif_addr >= 12'h80 + i0*12'h80) & (cpuif_addr <= 12'h80 + i0*12'h80 + 12'h53);
        end
        decoded_strb_is_external = is_external;
        external_req = is_external;
    end

    // Pass down signals to next stage
    assign decoded_addr = cpuif_addr;

    assign decoded_req = cpuif_req_masked;
    assign decoded_req_is_wr = cpuif_req_is_wr;
    assign decoded_wr_data = cpuif_wr_data;
    assign decoded_wr_biten = cpuif_wr_biten;

    //--------------------------------------------------------------------------
    // Field logic
    //--------------------------------------------------------------------------
    typedef struct {
        struct {
            struct {
                struct {
                    logic [31:0] next;
                    logic load_next;
                } n;
            } active_streams;
        } ctrl_regs;
    } field_combo_t;
    field_combo_t field_combo;

    typedef struct {
        struct {
            struct {
                struct {
                    logic [31:0] value;
                } n;
            } active_streams;
        } ctrl_regs;
    } field_storage_t;
    field_storage_t field_storage;

    // Field: tcp_cmd_regs.ctrl_regs.active_streams.n
    always_comb begin
        automatic logic [31:0] next_c;
        automatic logic load_next_c;
        next_c = field_storage.ctrl_regs.active_streams.n.value;
        load_next_c = '0;
        if(decoded_reg_strb.ctrl_regs.active_streams && decoded_req_is_wr) begin // SW write
            next_c = (field_storage.ctrl_regs.active_streams.n.value & ~decoded_wr_biten[31:0]) | (decoded_wr_data[31:0] & decoded_wr_biten[31:0]);
            load_next_c = '1;
        end
        field_combo.ctrl_regs.active_streams.n.next = next_c;
        field_combo.ctrl_regs.active_streams.n.load_next = load_next_c;
    end
    always_ff @(posedge clk) begin
        if(rst) begin
            field_storage.ctrl_regs.active_streams.n.value <= 32'h0;
        end else if(field_combo.ctrl_regs.active_streams.n.load_next) begin
            field_storage.ctrl_regs.active_streams.n.value <= field_combo.ctrl_regs.active_streams.n.next;
        end
    end
    assign hwif_out.ctrl_regs.active_streams.n.value = field_storage.ctrl_regs.active_streams.n.value;
    for(genvar i0=0; i0<16; i0++) begin
        assign hwif_out.tcp_streams[i0].req = decoded_reg_strb.tcp_streams[i0];
        assign hwif_out.tcp_streams[i0].addr = decoded_addr[6:0];
        assign hwif_out.tcp_streams[i0].req_is_wr = decoded_req_is_wr;
        assign hwif_out.tcp_streams[i0].wr_data = decoded_wr_data;
        assign hwif_out.tcp_streams[i0].wr_biten = decoded_wr_biten;
    end

    //--------------------------------------------------------------------------
    // Write response
    //--------------------------------------------------------------------------
    always_comb begin
        automatic logic wr_ack;
        wr_ack = '0;
        for(int i0=0; i0<16; i0++) begin
            wr_ack |= hwif_in.tcp_streams[i0].wr_ack;
        end
        external_wr_ack = wr_ack;
    end
    assign cpuif_wr_ack = external_wr_ack | (decoded_req & decoded_req_is_wr & ~decoded_strb_is_external);
    // Writes are always granted with no error response
    assign cpuif_wr_err = '0;

    //--------------------------------------------------------------------------
    // Readback
    //--------------------------------------------------------------------------
    logic readback_external_rd_ack_c;
    always_comb begin
        automatic logic rd_ack;
        rd_ack = '0;
        for(int i0=0; i0<16; i0++) begin
            rd_ack |= hwif_in.tcp_streams[i0].rd_ack;
        end
        readback_external_rd_ack_c = rd_ack;
    end

    logic readback_external_rd_ack;

    assign readback_external_rd_ack = readback_external_rd_ack_c;

    logic readback_err;
    logic readback_done;
    logic [31:0] readback_data;

    // Assign readback values to a flattened array
    logic [31:0] readback_array[18];
    assign readback_array[0][31:0] = (decoded_reg_strb.ctrl_regs.active_streams && !decoded_req_is_wr) ? field_storage.ctrl_regs.active_streams.n.value : '0;
    assign readback_array[1][31:0] = (decoded_reg_strb.ctrl_regs.max_streams && !decoded_req_is_wr) ? hwif_in.ctrl_regs.max_streams.fieldname.next : '0;
    for(genvar i0=0; i0<16; i0++) begin
        assign readback_array[i0*1 + 2] = hwif_in.tcp_streams[i0].rd_ack ? hwif_in.tcp_streams[i0].rd_data : '0;
    end

    // Reduce the array
    always_comb begin
        automatic logic [31:0] readback_data_var;
        readback_done = decoded_req & ~decoded_req_is_wr & ~decoded_strb_is_external;
        readback_err = '0;
        readback_data_var = '0;
        for(int i=0; i<18; i++) readback_data_var |= readback_array[i];
        readback_data = readback_data_var;
    end

    assign external_rd_ack = readback_external_rd_ack;
    assign cpuif_rd_ack = readback_done | readback_external_rd_ack;
    assign cpuif_rd_data = readback_data;
    assign cpuif_rd_err = readback_err;
endmodule
