import tcp_pkg::*;

module tcb_mem #(
    parameter N_TCP=16,
    localparam ADDR_WIDTH = $clog2(N_TCP),
    localparam TCB_WIDTH = $bits(tcb_t)
) (
    input  logic                    i_clk,
    input  logic                    i_rst,

    input  logic [7:0]              i_max_tcb,


    output logic [ADDR_WIDTH-1:0]   o_tcb_addr,
    output logic                    o_tcb_valid,
    input  logic                    i_tcb_ready,
    output tcb_t                    o_tcb,

    input  logic [ADDR_WIDTH-1:0]   i_tcb_addr,
    input  logic                    i_tcb_valid,
    input  tcb_t                    i_tcb
);

    logic tcb_mem_valid;

    logic [ADDR_WIDTH-1:0] tcb_addr, tcb_addr_next;

    logic [ADDR_WIDTH-1:0] tcb_addr_pipe[2];
    tcb_t tcb_pipe[2];
    logic [1:0] tcb_valid;

    tcp_dpram #(
        .ADDR_WIDTH(ADDR_WIDTH),
        .DATA_WIDTH(TCB_WIDTH)
    ) u_tcb_mem_ram (
        .clk0       (i_clk),        // Write Port
        .csb0       (~i_tcb_valid),
        .addr0      (i_tcb_addr),
        .din0       (i_tcb),
        .clk1       (i_clk),
        .csb1       (~i_tcb_ready),
        .addr1      (tcb_addr),
        .dout1      (tcb_pipe[0])
    );

    always_ff @(posedge i_clk) begin
        if (i_rst) begin
            tcb_addr <= '0;
            tcb_valid <= '0;
        end else begin
            tcb_addr <= tcb_addr_next;

            if (i_tcb_ready) begin
                tcb_valid <= {tcb_valid[0], i_tcb_ready};
                tcb_pipe[1] <= tcb_pipe[0];
                tcb_addr_pipe[0] <= tcb_addr;
                tcb_addr_pipe[1] <= tcb_addr_pipe[0];
            end
        end
    end

    always_comb begin
        o_tcb_valid = tcb_valid[1];
        o_tcb = tcb_pipe[1];
        o_tcb_addr = tcb_addr_pipe[1];

        tcb_addr_next = tcb_addr + {3'b0, i_tcb_ready};
    end


endmodule