module tcp_sw_interface #(
    parameter N_TCP=16,
    parameter EVENT_WIDTH=3,
    parameter EVENT_DATA_WIDTH=64,
    parameter TCP_ADDR_WIDTH=$clog2(N_TCP)
)(
    input  logic        i_clk,
    input  logic        i_rst,

    // APB
    input  logic [31:0] i_paddr,
    input  logic        i_psel,
    input  logic        i_penable,
    input  logic        i_pwrite,
    input  logic [31:0] i_pwdata,
    input  logic [3:0]  i_pstrb,
    output logic        o_pready,
    output logic [31:0] o_prdata,
    output logic        o_pslverr,

    output logic [TCP_ADDR_WIDTH-1:0] o_tcp_addr,
    output logic [EVENT_WIDTH-1:0] o_event,
    output logic [EVENT_DATA_WIDTH-1:0] o_event_data,
    output logic o_event_valid,
    input  logic i_event_ready
);

tcp_cmd_regs_pkg::tcp_cmd_regs__in_t tcp_cmd_hwif_in;
tcp_cmd_regs_pkg::tcp_cmd_regs__out_t tcp_cmd_hwif_out;

tcp_stream_regs_pkg::tcp_stream_regs__in_t tcp_stream_hwif_in[N_TCP];
tcp_stream_regs_pkg::tcp_stream_regs__out_t tcp_stream_hwif_out[N_TCP];

tcp_cmd_regs u_tcp_cmd_regs (
    .clk            (i_clk),
    .rst            (i_rst),

    .s_apb_psel     (i_psel),
    .s_apb_penable  (i_penable),
    .s_apb_pwrite   (i_pwrite),
    .s_apb_paddr    (i_paddr[11:0]),
    .s_apb_pwdata   (i_pwdata),
    .s_apb_pready   (o_pready),
    .s_apb_prdata   (o_prdata),
    .s_apb_pslverr  (o_pslverr),

    .hwif_in        (tcp_cmd_hwif_in),
    .hwif_out       (tcp_cmd_hwif_out)
);

generate
    for (genvar i = 0; i < N_TCP; i++) begin : stream_regs
        tcp_stream_regs u_tcp_stream_regs (
            .clk                    (i_clk),
            .rst                    (i_rst),

            .s_cpuif_req            (tcp_cmd_hwif_out.tcp_streams[i].req),
            .s_cpuif_req_is_wr      (tcp_cmd_hwif_out.tcp_streams[i].req_is_wr),
            .s_cpuif_addr           (tcp_cmd_hwif_out.tcp_streams[i].addr),
            .s_cpuif_wr_data        (tcp_cmd_hwif_out.tcp_streams[i].wr_data),
            .s_cpuif_wr_biten       (tcp_cmd_hwif_out.tcp_streams[i].wr_biten),
            .s_cpuif_req_stall_wr   (),
            .s_cpuif_req_stall_rd   (),
            .s_cpuif_rd_ack         (tcp_cmd_hwif_in.tcp_streams[i].rd_ack),
            .s_cpuif_rd_err         (),
            .s_cpuif_rd_data        (tcp_cmd_hwif_in.tcp_streams[i].rd_data),
            .s_cpuif_wr_ack         (tcp_cmd_hwif_in.tcp_streams[i].wr_ack),
            .s_cpuif_wr_err         (),

            .hwif_in                (tcp_stream_hwif_in[i]),
            .hwif_out               (tcp_stream_hwif_out[i])
        );
    end
endgenerate



endmodule