/*
 * Simple Dual Port RAM
 *
 * This ram is meant for FPGA, but shares the same interface with OpenRAM
 * generated rams such that the fifo core can be used in ASICs as well.
 *
 * (c) 2024 Byron Lathi
 */

module tcp_dpram #(
    parameter ADDR_WIDTH,
    parameter DATA_WIDTH
) (
  input  logic                  clk0, // clock
  input  logic                  csb0, // active low chip select
  input  logic [ADDR_WIDTH-1:0] addr0,
  input  logic [DATA_WIDTH-1:0] din0,
  input  logic                  clk1, // clock
  input  logic                  csb1, // active low chip select
  input  logic [ADDR_WIDTH-1:0] addr1,
  output logic [DATA_WIDTH-1:0] dout1
);

localparam RAM_DEPTH = 1 << ADDR_WIDTH;

logic [DATA_WIDTH-1:0] mem [RAM_DEPTH];

always @(posedge clk0) begin
    if (!csb0) begin
        mem[addr0] <= din0;
    end
end

always @(posedge clk1) begin
    if (!csb1) begin
        dout1 <= mem[addr1];
    end
end

endmodule

